@@ -17,7 +17,7 @@
  */
 
 /* Number of activities */
-#define NR_ACT	35
+#define NR_ACT	36
 
 /* Activities */
 #define A_CPU		1
@@ -55,6 +55,7 @@
 #define A_PWR_IN	33
 #define A_HUGE		34
 #define A_PWR_WGHFREQ	35
+#define A_PWR_USB	36
 
 
 /* Macro used to flag an activity that should be collected */
@@ -158,6 +159,7 @@
 #define K_SNMP		"SNMP"
 #define K_IPV6		"IPV6"
 #define K_POWER		"POWER"
+#define K_USB		"USB"
 
 /* Groups of activities */
 #define G_DEFAULT	0x00
@@ -186,6 +188,7 @@
 #define NR_SERIAL_PREALLOC	2
 #define NR_DISK_PREALLOC	3
 #define NR_FREQ_PREALLOC	0
+#define NR_USB_PREALLOC		5
 
 #define UTSNAME_LEN		65
 #define TIMESTAMP_LEN		16
@@ -687,7 +690,9 @@ extern __nr_t
 	wrap_get_in_nr(struct activity *);
 extern __nr_t
 	wrap_get_freq_nr(struct activity *);
-
+extern __nr_t
+	wrap_get_usb_nr(struct activity *);
+	
 /* Functions used to read activities statistics */
 extern __read_funct_t
 	wrap_read_stat_cpu(struct activity *);
@@ -759,6 +764,8 @@ extern __read_funct_t
 	wrap_read_meminfo_huge(struct activity *);
 extern __read_funct_t
 	wrap_read_time_in_state(struct activity *);
+extern __read_funct_t
+	wrap_read_bus_usb_dev(struct activity *);
 
 /* Other functions */
 extern void
